//----------------------------------------------------------------
//      _____
//     /     \
//    /____   \____
//   / \===\   \==/
//  /___\===\___\/  AVNET
//       \======/
//        \====/
//---------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2011 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------
//
// Create Date:         Sep 01, 2011
// Design Name:         FMC-IMAGEON
// Module Name:         fmc_imageon.h
// Project Name:        FMC-IMAGEON
// Target Devices:      Spartan-6, Virtex-6
//                      Artix-7, Kintex-7, Virtex-7, Zynq
// Avnet Boards:        FMC-IMAGEON
//
// Tool versions:       Vivado 2014.4
//
// Description:         FMC-IMAGEON Software Library.
//                      Includes functions for:
//                      - I2C Multiplexer
//                      - I2C I/O Expander
//                      - HDMI Input Configuration
//                      - HDMI Output Configuration
//
// Dependencies:
//
// Revision:            Sep 01, 2011: 1.01 Initial version
//                      Nov 14, 2011: 1.02 Add HDMI I2C config sequences
//                                         Add multiple I2C mux selections
//                      Dec 10, 2011: 1.03 Add HDMI functions
//                                         - fmc_imageon_hdmii_get_lock
//                                         - fmc_imageon_hdmii_get_timing
//                                         - fmc_imageon_hdmio_set_timing
//                      Feb 16, 2012: 1.04 Modify HDMI functions
//                                         - HDMI input
//                                            - I2C config for SPDIF
//                                         - HDMI output
//                                            - I2C config for SPDIF
//                                            - implement WaitForHPD
//                      Feb 21, 2012: 1.05 Modify HDMI output function
//                                         - set HDMI/DVI mode according
//                                           to pTiming argument
//                      Apr 13, 2012: 1.06 Fix typo in verbose
//                      Aug 15, 2012: 1.07 Set ADV7511 clock delay to 101 (+0.8ns)
//                      Nov 28, 2012: 2.01 Configure ADV7611 LLC for inverse polarity
//                                         for FMC-IMAGEON Rev.B hardware
//                      Jul 20, 2015: 2.02 Port to Vivado 2014.4
//
//----------------------------------------------------------------

#ifndef __FMC_IMAGEON_H__
#define __FMC_IMAGEON_H__

#include <stdio.h>

// Located in: microblaze_0/include/
#include "xparameters.h"
#include "xstatus.h"

#include "fmc_iic.h"

// Detailed ADV7611 I2C addresses
#define IIC_ADV7611_BASE_ADDR        0x98
#define IIC_ADV7611_CEC_ADDR         0x80
//#define IIC_ADV7611_INFOFRAME_ADDR   0x7C  => I2C Address conflict with ADV7511 (Fixed I2C Address at 0x7C)
#define IIC_ADV7611_INFOFRAME_ADDR   0x6A
#define IIC_ADV7611_DPLL_ADDR        0x4C
#define IIC_ADV7611_KSV_ADDR         0x64
#define IIC_ADV7611_EDID_ADDR        0x6C
#define IIC_ADV7611_HDMI_ADDR        0x68
#define IIC_ADV7611_CP_ADDR          0x44

// Detailed ADV7511 I2C addresses
#define IIC_ADV7511_BASE_ADDR        0x72


// FMC-IMAGEON I2C addresses
#define FMC_IMAGEON_I2C_MUX_ADDR  0x70 // 0xE0/0xE1 (PCA9546A)
#define FMC_IMAGEON_IO_EXP_ADDR   0x20 // 0x40/0x41 (PCA9534)
#define FMC_IMAGEON_VID_CLK_ADDR  0x65 // 0xCA/0xCB (CDCE913)
#define FMC_IMAGEON_HDMI_IN_ADDR  (IIC_ADV7611_BASE_ADDR>>1) // 0x98/0x99 (ADV7611)
#define FMC_IMAGEON_HDMI_OUT_ADDR (IIC_ADV7511_BASE_ADDR>>1) // 0x72/0x73 (ADV7511)
#define FMC_IMAGEON_DDCEDID_ADDR  0x50 // 0xA0/0xA1


struct struct_fmc_imageon_t
{
   // software library version
   u32 uVersion;

   // instantiation-specific name
   char szName[32];

   // pointer to FMC-IIC instance
   fmc_iic_t *pIIC;

   // GPIO value
   u32 GpioData;

   // Verblse
   u32 bVerbose;
};
typedef struct struct_fmc_imageon_t fmc_imageon_t;

struct struct_fmc_imageon_video_timing_t
{
   //char *pName;

   // General info
   u32 IsHDMI;
   u32 IsEncrypted;
   u32 IsInterlaced;
   u32 ColorDepth;

   // Horizontal Timing
   u32 HActiveVideo;
   u32 HFrontPorch;
   u32 HSyncWidth;
   u32 HBackPorch;
   u32 HSyncPolarity;

   // Vertical Timing
   u32 VActiveVideo;
   u32 VFrontPorch;
   u32 VSyncWidth;
   u32 VBackPorch;
   u32 VSyncPolarity;

   float tmds_freq;
};
typedef struct struct_fmc_imageon_video_timing_t fmc_imageon_video_timing_t;


int fmc_imageon_init( fmc_imageon_t *pContext, char szName[], fmc_iic_t *pIIC );

// I2C MUX Functions
void fmc_imageon_iic_mux_reset( fmc_imageon_t *pContext );
void fmc_imageon_iic_mux( fmc_imageon_t *pContext, u32 MuxSelect );
// Single Mux Selections
#define FMC_IMAGEON_I2C_MIN               0
#define FMC_IMAGEON_I2C_MAX               3
#define FMC_IMAGEON_I2C_SELECT_DDCEDID    0
#define FMC_IMAGEON_I2C_SELECT_HDMI_OUT   1
#define FMC_IMAGEON_I2C_SELECT_HDMI_IN    2
#define FMC_IMAGEON_I2C_SELECT_IO_EXP     3
#define FMC_IMAGEON_I2C_SELECT_VID_CLK    3
#define FMC_IMAGEON_I2C_SELECT_CAM        5
// Multiple Mux Selections
#define FMC_IMAGEON_I2C_SELECT_HDMI       4 // select both HDMI_IN and HDMI_OUT

// General I2C Configuration Functions
void fmc_imageon_iic_config2( fmc_imageon_t *pContext, u8 ChipAddress,
                              u8 ConfigData[][2], u32 ConfigLength );
void fmc_imageon_iic_config3( fmc_imageon_t *pContext,
                              u8 ConfigData[][3], u32 ConfigLength );

// Video Clock Synthesizer Functions
void fmc_imageon_vclk_init( fmc_imageon_t *pContext );
void fmc_imageon_vclk_config( fmc_imageon_t *pContext, u32 FreqId );
#define FMC_IMAGEON_VCLK_FREQ_25_175_000       0
#define FMC_IMAGEON_VCLK_FREQ_27_000_000       1
#define FMC_IMAGEON_VCLK_FREQ_40_000_000       2
#define FMC_IMAGEON_VCLK_FREQ_65_000_000       3
#define FMC_IMAGEON_VCLK_FREQ_74_250_000       4
#define FMC_IMAGEON_VCLK_FREQ_110_000_000      5
#define FMC_IMAGEON_VCLK_FREQ_148_500_000      6
#define FMC_IMAGEON_VCLK_FREQ_162_000_000      7

//LKO
#define FMC_IMAGEON_VCLK_FREQ_IN        27.0
#define FMC_IMAGEON_VCLK_FREQ_125MHZ    125.0
#define FMC_IMAGEON_VCLK_FREQ_150MHZ    150.0
#define FMC_IMAGEON_VCLK_FREQ_175MHZ    175.0
#define FMC_IMAGEON_VCLK_FVCO_MIN       80.0
#define FMC_IMAGEON_VCLK_FVCO_MAX       230.0
#define FMC_IMAGEON_VCLK_FVCO_MIN_ERR   0.001

struct cdce913_pll_conf {
    struct {
        u32 vco_range : 2;
        u32 p         : 3;
        u32 q         : 6;
        u32 r         : 9;
        u32 m         : 9;
        u32 n         : 12;
        u32 pdiv      : 10;
    };
    u8 err;
    u8 data[6];
};

double my_fabs(double x);
double my_floor(double x);
double my_modf(double x, double *integer);
struct cdce913_pll_conf cdce913_find_coeffs(double fvco);
int fmc_imageon_vclk_set(fmc_imageon_t *pContext, double freq_mhz);


// HDMI Input Functions
int fmc_imageon_hdmii_init( fmc_imageon_t *pContext, u32 Enable, u32 edidInit, u8 pEdid[256] );
int fmc_imageon_hdmii_init2( fmc_imageon_t *pContext, u32 Enable, u32 edidInit, u8 pEdid[256], u32 llc_polarity, u32 llc_delay );
int fmc_imageon_hdmii_set_hpd( fmc_imageon_t *pContext, u32 HotPlugStatus );
int fmc_imageon_hdmii_set_rst( fmc_imageon_t *pContext, u32 Reset );
int fmc_imageon_hdmii_get_int( fmc_imageon_t *pContext, u32 *pIntStatus );
int fmc_imageon_hdmii_get_lock( fmc_imageon_t *pContext );
int fmc_imageon_hdmii_get_timing( fmc_imageon_t *pContext, fmc_imageon_video_timing_t *pTiming );

// HDMI Output Functions
int fmc_imageon_hdmio_init( fmc_imageon_t *pContext, u32 Enable, fmc_imageon_video_timing_t *pTiming, u32 WaitForHPD );
int fmc_imageon_hdmio_set_pd( fmc_imageon_t *pContext, u32 PowerDown );
int fmc_imageon_hdmio_get_hpd( fmc_imageon_t *pContext, u32 *pHotPlugDetect );

// DDC/EDID Functions
int fmc_imageon_hdmii_read_edid( fmc_imageon_t *pContext, u8 data[256] );
int fmc_imageon_hdmii_write_edid( fmc_imageon_t *pContext, u8 data[256] );
int fmc_imageon_hdmio_read_edid( fmc_imageon_t *pContext, u8 data[256] );

// Delay Functions
void fmc_imageon_wait_usec(u32 delay);

#endif // __FMC_IMAGEON_H__
